%bcond_with shared_libuv
%bcond_with shared_cares
%bcond_with shared_brotli
%bcond_with shared_icu

%define major 20
%define minor 11
%define fix 1

%define node_prefix %{_libdir}/%{name}

Name: nodejs%{major}
Version: %{major}.%{minor}.%{fix}
Release: 1
License: MIT
Summary: Node.js JavaScript Runtime
Url: https://www.nodejs.org

BuildRequires: pkg-config
BuildRequires: python39
BuildRequires: zlib-devel
BuildRequires: openssl-3.0-devel
%{?with_shared_icu:BuildRequires: icu-devel}
%{?with_shared_cares:BuildRequires: c-ares-devel}
%{?with_shared_libuv:BuildRequires: libuv-devel}
%{?with_shared_brotli:BuildRequires: brotli-devel}

BuildRequires: gcc10
BuildRequires: gcc10-cplusplus
BuildRequires: libstdcplusplus6-10-devel >= 10.3.0

Requires: libopenssl3
Requires: python39
Requires: update-alternatives

Requires: libstdcplusplus6 >= 10.3.0
Requires: libgcc_s1 >= 10.3.0

Requires(post): update-alternatives
Requires(postun): update-alternatives

Conflicts: nodever < 1.0.0
Source0: https://nodejs.org/dist/v%{version}/node-v%{version}.tar.xz

%description
Node.js is a JavaScript runtime built on Chrome V8 JavaScript engine. Node.js
uses an event-driven, non-blocking I/O model that makes it lightweight and
efficient.

%prep
%setup -q -n node-v%{version}

# Fix the wrong dependency /usr/bin/python
find deps tools ./configure* -type f | xargs perl -pi -e 's|/usr/bin/python|%{_bindir}/python3.9|g'

%build

export CC=gcc-10
export CXX=g++-10

./configure \
  --prefix=%{node_prefix} \
  --dest-cpu=ppc64 \
  --without-node-snapshot \
  --shared-openssl \
  --shared-zlib \
   %{?with_shared_icu:--with-intl=system-icu} \
   %{?with_shared_cares:--shared-cares} \
   %{?with_shared_libuv:--shared-libuv} \
   %{?with_shared_brotli:--shared-brotli} \
  # end

%make_build
# basic verification
./node -e "console.log(process.versions)"

%check

make test-only

%install

%make_install V=1 PORTABLE=1

# Create node headers tar file for use with npm
# otherwise, npm will download and use the unpatched versions :(
#
# NOTE: This can be removed once npm changes have been merged upstream.
cd %{buildroot}%{node_prefix}
tar -czf node-headers.tar.gz ./include
 
# Create an "npm builtin config file" to point npm at the header tarball
# created above. If users want to override, they can do so using the per-user,
# or per-project config files:
# https://docs.npmjs.com/cli/v8/configuring-npm/npmrc#files
mkdir ./etc
cat <<EOF > ./etc/npmrc
tarball=%{node_prefix}/node-headers.tar.gz
script-shell=%{_bindir}/bash
EOF

%post
###############################################################################
# processing after the install
# $1 = 1: install
# $1 = 2+: upgrade
###############################################################################

type=upgrade
[ $1 -eq 1 ] && type=install

if [ $type = 'install' ]
then
  # set up the just installed version of Node.js in update-alternatives
  %{_bindir}/alternatives --install %{_bindir}/node node %{_libdir}/nodejs%{major}/bin/node %{major} \
  --slave %{_bindir}/npm npm %{_libdir}/nodejs%{major}/bin/npm \
  --slave %{_bindir}/npx npx %{_libdir}/nodejs%{major}/bin/npx
fi


%postun
###############################################################################
# processing after the uninstall
# $1 = 0: uninstall
# $1 = 1: upgrade
###############################################################################

type=uninstall
[ $1 -eq 1 ] && type=upgrade

# After uninstallation, need to check if it was truly an uninstallation, and if
# so, remove the the update-alternatives entry for this version of node
if [ $type = 'uninstall' ]
then
  %{_bindir}/alternatives --remove node %{_libdir}/nodejs%{major}/bin/node
fi


%files
# Set the default owner to QSYS and group to *NONE
%defattr(-, qsys, *none)

%{node_prefix}

%changelog
* Wed Feb 14 2024 Abdirahim Musse <abdirahim.musse@ibm.com> - 20.11.1
- Update to 20.11.1
- https://github.com/nodejs/node/releases/tag/v20.11.1
- Remove v8 deps patch as it was upstreamed
- https://github.com/nodejs/node/commit/7ca1228be8

* Fri Oct 13 2023 Abdirahim Musse <abdirahim.musse@ibm.com> - 20.8.1
- Update for security release
- https://github.com/nodejs/node/releases/tag/v20.8.1
- https://nodejs.org/en/blog/vulnerability/october-2023-security-releases

* Wed Aug 9 2023 Abdirahim Musse <abdirahim.musse@ibm.com> - 20.5.1
- Update for security release
- https://github.com/nodejs/node/releases/tag/v20.5.1
- https://nodejs.org/en/blog/vulnerability/august-2023-security-releases

* Mon Jul 24 2023 Abdirahim Musse <abdirahim.musse@ibm.com> - 20.5.0
- Update to 20.5.0
- https://github.com/nodejs/node/releases/tag/v20.5.0

* Tue Jun 20 2023 Abdirahim Musse <abdirahim.musse@ibm.com> - 20.3.1
- Update for security release
- https://nodejs.org/en/blog/vulnerability/june-2023-security-releases

* Mon Apr 24 2023 Abdirahim Musse <abdirahim.musse@ibm.com>
- Build with openssl-3.0-devel
- Drop patches that got upstreamed

* Wed Feb 22 2023 Xu Meng <mengxumx@cn.ibm.com>
- BuildRequire openssl-1.1-devel instead of openssl-devel

* Thu Feb 2 2023 Xu Meng <mengxumx@cn.ibm.com>
- Initial package for verification
